/* $Id: bfd.c,v 1.2 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************

	This is a wrapper to the libbfd-mips-ecoff.a library. libbfd is easy to
	use, but wastes resources.

	Because libbfd has its own file management, the mmap'ed buffer in
	``pio_file_t *file'' is always ignored. But since linux is very clever,
	this doesn't matter very much... :-)

 */

#include "defs.h"
#include "pio.h"
#include "rom.h"
#include "error.h"
#include "yaroze.h"

#if !defined(__NetBSD__)
#include <ansidecl.h>
#include <bfd.h>
#else
#include <bfd/ansidecl.h>
#include <bfd/bfd.h>
#endif

static bool bfd_detect (pio_file_t *file)
{
	bfd *abfd;
	bool detected = FALSE;

	abfd = bfd_openr(file->name,0);
	if (abfd != NULL)
	{
		if (bfd_check_format(abfd,bfd_object))
		{
			verbosef (VERBOSE_CHECK,"detected %s\n",abfd->xvec->name);
			detected = TRUE;
		}
		bfd_close(abfd);
	}

	return detected;
}

static bool bfd_send (pio_file_t *file)
{
	bfd *abfd;

	abfd = bfd_openr (file->name,0);
	if (abfd != NULL)
	{
		if (bfd_check_format(abfd,bfd_object))
		{
			asection *section;
			bfd_size_type datasize = 0;

			verbosef (VERBOSE_ALL,
				"Object file format is \"%s\"\n"
				"PC=0x%08lx GP=0x%08lx\n"
				,abfd->xvec->name
				,abfd->start_address
				,bfd_ecoff_get_gp_value(abfd));

			for (section = abfd->sections; section != NULL; section = section->next)
			{
				datasize = bfd_section_size (abfd,section);
				
				verbosef (VERBOSE_PROGRESS,"Section [%8s] Address [%08lx] Length [%06lx]",section->name,section->vma,datasize);
				if (section->flags & SEC_HAS_CONTENTS)
				{
					char *data = (bfd_byte*) malloc (datasize);
					
					verbosef (VERBOSE_PROGRESS," * ");
					
					bfd_get_section_contents (abfd,section,data,0,datasize);
					if (datasize)
					rom_call (send_memory,section->vma,data,datasize);

					free (data);
				}
				verbosef (VERBOSE_PROGRESS,"\n");
			}

			if (flags & FLAG_YAROZE)
		 	{
				u_long ny_patch[sizeof (ny_patch_org) /
						sizeof (ny_patch_org[0])];

				memcpy(ny_patch, ny_patch_org, sizeof (ny_patch));
				*(u_short *)((void *)ny_patch + PATCH_PC_OFFSETH) = abfd->start_address >> 16;
				*(u_short *)((void *)ny_patch + PATCH_PC_OFFSETL) = abfd->start_address & 0xffff;

				verbosef (VERBOSE_ALL, "Patch to NetYaroze Exec\n");
				rom_call(send_memory, (u_long)PATCH_ADDR, (u_char *)ny_patch, sizeof (ny_patch));
			}
		}
		else warnf ("bfd_check_format(); %s\n",bfd_errmsg(bfd_get_error()));
		bfd_close (abfd);
	}
	else
	{
		warnf ("bfd_openr(%s); %s\n",file->name,bfd_errmsg(bfd_get_error()));
		return FALSE;
	}
	
	return TRUE;
}

static bool bfd_get_header (pio_file_t *file, ps_header_t *h)
{
	bfd *abfd;

	h->pc0		= 0;	h->gp0		= 0;
	h->t_addr	= 0;	h->t_size	= 0;
	h->d_addr	= 0;	h->d_size	= 0;
	h->b_addr	= 0;	h->b_size	= 0;
	h->s_addr	= 0;	h->s_size	= 0;
	h->sp		= 0;	h->fp		= 0;
	h->gp		= 0;	h->ret		= 0;
	h->base		= 0;

	abfd = bfd_openr (file->name,0);
	if (abfd != NULL)
	{
		if (bfd_check_format(abfd,bfd_object))
		{
			asection *section;

			if (flags & FLAG_YAROZE)
				h->pc0 = PATCH_ADDR;
			else
				h->pc0 = abfd->start_address;
			h->gp0 = bfd_ecoff_get_gp_value (abfd);

			section = bfd_get_section_by_name (abfd,".text");
			if (section)
			{
				h->t_size = bfd_section_size (abfd,section);
				h->t_addr = section->vma;
			}

			section = bfd_get_section_by_name (abfd,".data");
			if (section)
			{
				h->d_size = bfd_section_size (abfd,section);
				h->d_addr = section->vma;
			}

			section = bfd_get_section_by_name (abfd,".bss");
			if (section)
			{
				h->b_size = bfd_section_size (abfd,section);
				h->b_addr = section->vma;
			}
		}
		else warnf ("bfd_check_format(); %s\n",bfd_errmsg(bfd_get_error()));
		bfd_close (abfd);
	}
	else
	{
		warnf ("bfd_openr(%s); %s\n",file->name,bfd_errmsg(bfd_get_error()));
		return FALSE;
	}

	return TRUE;
}

static bool bfd_dump (pio_file_t *file)
{
	bfd *abfd;

	abfd = bfd_openr (file->name,0);
	if (abfd != NULL)
	{
		if (bfd_check_format(abfd,bfd_object))
		{
			asection *section;
			bfd_size_type datasize = 0;

			verbosef (VERBOSE_ALL,
				"Object file format is \"%s\"\n"
				"PC=0x%08lx GP=0x%08lx\n"
				,abfd->xvec->name
				,abfd->start_address
				,bfd_ecoff_get_gp_value(abfd));

			for (section = abfd->sections; section != NULL; section = section->next)
			{
				datasize = bfd_section_size (abfd,section);
				
				verbosef (VERBOSE_ALL,"Section [%8s] Address [%08lx] Length [%06lx]",section->name,section->vma,datasize);
				if (section->flags & SEC_HAS_CONTENTS) verbosef (VERBOSE_ALL," * ");
				verbosef (VERBOSE_ALL,"\n");
			}
		}
		else warnf ("bfd_check_format(); %s\n",bfd_errmsg(bfd_get_error()));
		bfd_close (abfd);
	}
	else
	{
		warnf ("bfd_openr(%s); %s\n",file->name,bfd_errmsg(bfd_get_error()));
		return FALSE;
	}

	return TRUE;
}

pio_format_t bfd_pio = {
	NULL,
	
	// module informations	
	"BFD",
	"0.1",
	"BFD format driver",
	"Daniel Balster <dbalster@uni-paderborn.de>",
	"Daniel Balster <dbalster@uni-paderborn.de>",

	bfd_init,
	bfd_detect,
	bfd_send,
	bfd_get_header,
	bfd_dump,
};
