/* $Id: caetla.c,v 1.5 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998,1999 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************

	This is the wrapper for the CAETLA rom. It isn't finished yet and maybe
	needs a lot of change.

	It consists of three parts: support functions, module functions and the
	file server part.

 */

#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <errno.h>
#include <fnmatch.h>
#include "rom.h"
#include "error.h"

static u_char buf [1024];
static u_char path [1024];
static u_char file [1024];
static DIR * dirstream;

static u_char caetla_server_quit = 0;
static u_char caetla_alarm_set = 0;

/* Booboo */
static void control_c()
{
	caetla_server_quit = 1;
}

/* Booboo */
static void caetla_alarm()
{
	caetla_alarm_set = 1;
}

///////////////////////////////////////////////////////////////////////////
// caetla functions

static void caetla_command (u_char command)
{
	u_char x='6';
	void *old_sig;

	caetla_alarm_set = 0;
	old_sig = signal(SIGALRM, caetla_alarm);
	alarm(3); 
	while (caetla_alarm_set == 0)
	{
		x = rom_swap_8('C');
		if (x == 'd') break;
	}

	if (caetla_alarm_set) errorf ("protocol error\n");

	x = rom_swap_8('L');
	if (x != 'o') errorf ("protocol error\n");

	x = rom_swap_8 (command);
	if (x != 0) errorf ("protocol error\n");
}

static bool caetla_detect (void)
{
	u_char x;
	void *old_sig;

	caetla_alarm_set = 0;
	old_sig = signal(SIGALRM, caetla_alarm);
	alarm(3);
	while (caetla_alarm_set == 0)
	{
		x = rom_swap_8('C');
		if (rom_error) return FALSE;
		if (x == 'd') break;
	}
	signal(SIGALRM, old_sig);
	if (caetla_alarm_set) return FALSE;

	x = rom_swap_8('L');
	if (rom_error) return FALSE;
	if (x != 'o') return FALSE;

	x = rom_swap_8 (13);
	if (rom_error) return FALSE;
	if (x != 0) return FALSE;

	return TRUE;
}

/*
**	
*/
static bool caetla_listen (void)
{
	u_char x;

	do {
		caetla_command (0x01);
		x = rom_swap_8(rom_swap_8(0));
	} while (x == 1);

	if (x & 0x80) errorf ("caetla protocol error\n");

	caetla_command (0x13);

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////
// server functions

static void caetla_con_putchar (void)
{
	putchar (rom_swap_8(0));
	fflush (stdout);
}

static void caetla_con_getchar (void)
{
	rom_swap_8 (getchar());
}

static void caetla_con_open (void)
{
	u_char mode;
	int rc;
	int i;

	rom_get_string (buf);

        i=0;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      buf[i] = '/';
            i++;
	  }

	mode = rom_swap_8(0); /* bah, mode seems to be always 2 */
	rc = open (buf, O_RDWR);
	if (rc<0) /* fallback to O_RDONLY if O_RDWR fails */
	  rc = open (buf, O_RDONLY);
	if (rc<0) /* if that fails too, report error */
	  perror("open");
	rom_swap_8 (rc < 0 ? 0xff : 0x00);
	rom_swap_16 (rc);
}

static void caetla_con_create (void)
{
	u_char mode;
	int rc;
	int i;

	rom_get_string (buf);

        i=0;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      buf[i] = '/';
            i++;
	  }

	rc = creat (buf, 0644);
	if (rc<0)
	  perror("creat");
	mode = rom_swap_8(0);
	rom_swap_8 (rc < 0 ? 0xff : 0x00);
	rom_swap_16 (rc);
}

static void caetla_con_read (void)
{
	int fd, fd2, r;
	u_long size;
	u_char x = 0;

	fd = rom_swap_8(0);
	fd2 = rom_swap_8(0);
	size = rom_swap_32(0);
	while (size) {
		r = size > 1024 ? 1024 : size;
		if (fd)
		  r = read (fd, buf, r);
		else
		  r = read (fd2, buf, r);
		if (r <= 0) { x = 0; break; }
		rom_swap_32(r);
		x = rom_send_memory (buf, r);
		if (x != rom_swap_8(0)) { x = 0xff; break; }
		size -= r;
		x = 0;
	}
	rom_swap_32(0);
	rom_swap_8(x);
}

static void caetla_con_write (void)
{
	int fd, fd2, r;
	u_long size;
	u_char x = 0;

	fd = rom_swap_8(0);
	fd2 = rom_swap_8(0);
	size = rom_swap_32(0);
	while (size) {
		r = size > 1024 ? 1024 : size;
		rom_swap_32(r);
		x = rom_recv_memory(buf, r);
		if (x != rom_swap_8(0)) { x = 0xff; break; }
		if (fd)
		  write(fd, buf, r);
		else
		  write(fd2, buf, r);
		size -= r;
		x = 0;
	}
	rom_swap_32(0);
	rom_swap_8(x);
}

static void caetla_con_seek (void)
{
	u_long rc;
	int fd,fd2;
	off_t offset;
	int whence;

        fd=rom_swap_8(0);
	fd2=rom_swap_8(0);
	offset=rom_swap_32(0);
	whence=rom_swap_8(0);

	if(fd)
	  rc = lseek (fd, offset, whence);
	else
	  rc = lseek (fd2, offset, whence);

	rom_swap_8(0);
	rom_swap_32(rc);

	if(rc==(off_t)-1)
	  perror("lseek");
}

static void caetla_con_close (void)
{
        int fd, fd2;

        fd = rom_swap_8(0);
	fd2 = rom_swap_8(0);
	if (fd)
	  close (fd);
	else
	  close (fd2);
	rom_swap_8(0);
}

static void caetla_con_delete (void)
{
        int i;

	rom_get_string (buf);

        i=0;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      buf[i] = '/';
            i++;
	  }

	if (unlink(buf) < 0)
	        rom_swap_8(0xff);
		else
		rom_swap_8(0x00);
}

static void caetla_con_rename (void)
{
        int i;

	rom_get_string (buf);

        i=0;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      buf[i] = '/';
            i++;
	  }

	rom_get_string (&buf[512]);

        i=512;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      buf[i] = '/';
            i++;
	  }

	if (rename(buf,&buf[512]) < 0)
		rom_swap_8(0xff);
	else
		rom_swap_8(0x00);
}

static void caetla_con_files (void)
{
	int i,j;
	struct dirent * direntry;
	struct stat filestat;
	unsigned char successful=0;
	unsigned char temp[1024] = "./";
	unsigned char attr=0;
	struct tm * thetime;

	rom_swap_16 (0);

// get filename
	rom_get_string (buf);
/* unixify it */

        i=0;
	j=0;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      {
		buf[i] = '/';
		j=i;
	      }
            i++;
	  }

	/* separate out the path */
	memset(path, 0, 1024);
	strncpy(path, "./", 3);
	strncpy(&path[2], buf, j);
	path[j+3]=0;

        /* separate out the filename */
	memset(file, 0, 1024);
	if (buf[j]=='/')
	  strncpy(file, &buf[j+1], i-(j+1));
	else
	  strncpy(file, &buf[j], i-j);
	  
        dirstream = opendir(path);

	if (dirstream == NULL)
	  {
	    perror("opendir");
	    successful=0xff;
	  }
	else
	  {
	    direntry = readdir(dirstream);
	    successful=0xff;
	    while ((direntry != NULL)&&(successful==0xff))
	      {
		if(fnmatch(file, direntry->d_name, 0)==0)
		  {
		    successful=0;
		    strncpy(temp, path, 1024);
		    strncat(temp, "/", 1024);
		    strncat(temp, direntry->d_name, 1024);
		    stat(temp, &filestat);
		  }
		else
		  direntry = readdir(dirstream);
	      }
	  }

	/* MSDOS filesystem crap expected by Caetla 
	 *
	 * attribute      7  6  5  4  3  2  1  0
	 *                X  X  |  |  |  |  |  readonly
	 *                      |  |  |  |  hidden 
	 *                      |  |  |  system
	 *                      |  |  label
	 *                      |  subdir
	 *                      archive
	 *
	 * time           F E D C B   A 9 8 7 6 5   4 3 2 1 0
	 *                hours       minutes       seconds (2 sec. precision)
	 *
	 * date           F E D C B A 9   8 7 6 5   4 3 2 1 0
	 *                year-1980       month     day
	 *
	 */
	
	if (filestat.st_mode&&S_IFDIR)
	  attr|=0x10; /* subdir is a useful attribute */
	
	thetime = localtime(&filestat.st_mtime);

        rom_swap_8(attr); /* attribute */
	rom_swap_16((thetime->tm_hour<<11)|(thetime->tm_min<<5)|(thetime->tm_sec>>2)); /* time */
	rom_swap_16(((thetime->tm_year-1980)<<9)|(thetime->tm_mon<<5)|(thetime->tm_mday<<9)); /* date */

	/* size is an oddball - opposite byte order from everything else */
	rom_swap_8(filestat.st_size&0xff); /* size */
	rom_swap_8((filestat.st_size&0xff00)>>8); /* size */
	rom_swap_8((filestat.st_size&0xff0000)>>16); /* size */
	rom_swap_8((filestat.st_size&0xff000000)>>24); /* size */

	if (successful==0x00)
	  for (i=0;i<13;i++) rom_swap_8(direntry->d_name[i]);
	else
	  for (i=0;i<13;i++) rom_swap_8(0);
	rom_swap_8(successful);
}

static void caetla_con_nfiles (void)
{
        int successful=0xff;
	int i;
	struct dirent * direntry;
	struct stat filestat;
	unsigned char temp[1024];
	unsigned char attr=0;
	struct tm * thetime;

        direntry = readdir(dirstream);

	while ((direntry != NULL)&&(successful==0xff))
	  {
	    if(fnmatch(file, direntry->d_name, 0)==0)
	      {
		successful=0;
		strncpy(temp, path, 1024);
		strncat(temp, "/", 1024);
		strncat(temp, direntry->d_name, 1024);
		stat(temp, &filestat);
	      }
	    else
	      direntry = readdir(dirstream);
	  }

	if (filestat.st_mode&&S_IFDIR)
	  attr|=0x10;

	thetime = localtime(&filestat.st_mtime);

        rom_swap_8(attr); /* attribute */
	rom_swap_16((thetime->tm_hour<<11)|(thetime->tm_min<<5)|(thetime->tm_sec>>2)); /* time */
	rom_swap_16(((thetime->tm_year-1980)<<9)|(thetime->tm_mon<<5)|(thetime->tm_mday<<9)); /* date */

	/* size is an oddball, opposite byte order from everything else */

	rom_swap_8(filestat.st_size&0xff); /* size */
	rom_swap_8((filestat.st_size&0xff00)>>8); /* size */
	rom_swap_8((filestat.st_size&0xff0000)>>16); /* size */
	rom_swap_8((filestat.st_size&0xff000000)>>24); /* size */

	if (successful==0x00)
	  for (i=0;i<13;i++) rom_swap_8(direntry->d_name[i]);
	else
	  for (i=0;i<13;i++) rom_swap_8(0);

	rom_swap_8(successful);
}

static void caetla_con_chdir (void)
{
        int i;

	rom_get_string (buf);

        i=0;
	while (buf[i] != 0)
	  {
	    if (buf[i] == '\\')
	      buf[i] = '/';
            i++;
	  }

	if (chdir(buf) < 0)
		rom_swap_8(0xff);
	else
		rom_swap_8(0x00);
}

static void caetla_con_curdir (void)
{
        unsigned char cd[1024];
        int i;

        getcwd(cd, 1024);

        i=0;
	while (cd[i] != 0)
	  {
	    if (cd[i] == '/')
	      cd[i] = '\\';
            i++;
	  }

	rom_swap_8(0); /* the drive, but we don't give a rat's ass */
	rom_put_string(cd);
	rom_swap_8(0);
}

static void caetla_con_chdrv (void)
{
        rom_swap_8(0); /* the drive to change to */
	rom_swap_8(0); /* the number of devices */
}

static void caetla_con_curdrv (void)
{
        rom_swap_8(0); /* the current drive */
}

static call_t caetla_con[] = {
	NULL,
	caetla_con_putchar, caetla_con_getchar, caetla_con_open,
	caetla_con_create,  caetla_con_read,    caetla_con_write,
	caetla_con_seek,    caetla_con_close,   caetla_con_delete,
	caetla_con_rename,  caetla_con_files,   caetla_con_nfiles,
	caetla_con_chdir,   caetla_con_curdir,  caetla_con_chdrv,
	caetla_con_curdrv
};

///////////////////////////////////////////////////////////////////////////
// module functions

static void caetla_reset (void)
{
	caetla_command (8);
	
	switch (rom_swap_8(0))
	{
		case 0x00:
			break;
		case 0x02:	// geemu ### koodo
		case 0x04:
			caetla_command (3); // ps no risetto
			caetla_command (0);	// nop
			break;
	}
}

/*
	start executable
*/

static void caetla_boot (char *filename, ps_env_t *e, bool console)
{
}

static void caetla_send_memory (u_long address, u_char *buffer, u_long size)
{
        u_long pcchk,psxchk;
  
	assertz (buffer,"buffer");
	assertz (size,"size");

	caetla_listen();

	caetla_command (0x14);
	rom_swap_32 (address);
	rom_swap_32 (size);
	pcchk=rom_send_memory (buffer,size);
	psxchk=rom_swap_8 (0);
	if (pcchk!=psxchk)
	  printf("Checksum failure\n");
}

static void caetla_recv_memory (u_long address, u_char *buffer, u_long size)
{
	assertz (buffer,"buffer");
	assertz (size,"size");

	caetla_listen();

	caetla_command (0x15);
	rom_swap_32 (address);
	rom_swap_32 (size);
	rom_recv_memory (buffer,size);
	rom_swap_8 (0);
}

static void caetla_send_vram (vram_image_t *image)
{
	int i;

	caetla_command (0x01);			// select caetla menu 2
	rom_swap_8 (2);
	
	verbosef (VERBOSE_PROGRESS,
	"Uploading VRAM %dx%d+%d+%d in %d-Bit\n",
	image->w,image->h,image->x,image->y,image->f ? 24:16);

	if (image->pixel != NULL)
	{
		caetla_command (0x12);			// send vram
		rom_swap_16 (image->x);
		rom_swap_16 (image->y);
		rom_swap_16 (image->w);
		rom_swap_16 (image->h);
	
		verbose_if (VERBOSE_PROGRESS) progress_init (image->w*image->h);
		for (i=0;i<(image->w*image->h);i++)
		{
			rom_swap_16 (SWAP16(image->pixel[i]));
			verbose_if (VERBOSE_PROGRESS) if ((i % image->w)==0) progress_bar (i);
		}
		verbose_if (VERBOSE_PROGRESS) progress_end ();
	}

	caetla_command (0x02);			// back to main menu ??
}

/*
	not yet finished, only base parts.
*/

static void caetla_recv_vram (vram_image_t *image)
{
	u_short x,y,w,h,f,fix;
	int i;

//	caetla_command (0x01);			// select caetla menu 2 ??
//	rom_swap_8 (2);
	
	caetla_command (0x13);			// get vram parameter
	x = rom_swap_16 (0);
	y = rom_swap_16 (0);
	w = rom_swap_16 (0);
	h = rom_swap_16 (0);
	f = rom_swap_8 (0);

	if (image->x == 0xffff) image->x = x;
	if (image->y == 0xffff) image->y = y;
	if (image->w == 0xffff) image->w = w;
	if (image->h == 0xffff) image->h = h;
	if (image->f == 0xffff) image->f = f;

	verbosef (VERBOSE_PROGRESS,
	"Downloading VRAM %dx%d+%d+%d in %d-Bit\n",
	image->w,image->h,image->x,image->y,image->f ? 24:16);

// do a check here !!

	image->pixel = malloc (image->w*image->h*(2+image->f));

	fix = image->f ? (image->w/2) : 0;

	if (image->pixel != NULL)
	{
		caetla_command (0x11);			// set vram parameter
		rom_swap_16 (image->x);
		rom_swap_16 (image->y);
		rom_swap_16 (image->w+fix);
		rom_swap_16 (image->h);
	
		if (image->f==1)		// 24-bit
		{
			u_char *bytes = (u_char *) image->pixel;
		
			for (i=0;i<(image->w*image->h);i++)
			{
				*bytes = rom_swap_8 (0); bytes++;
				*bytes = rom_swap_8 (0); bytes++;
				*bytes = rom_swap_8 (0); bytes++;
			}
		}
		else				// 16-bit
		{
			verbose_if (VERBOSE_PROGRESS) progress_init (image->w*image->h);
			for (i=0;i<(image->w*image->h);i++)
			{
				u_short w = rom_swap_16(0);
				image->pixel[i] = SWAP16(w);
				verbose_if (VERBOSE_PROGRESS) if ((i % image->w)==0) progress_bar (i);
			}
			verbose_if (VERBOSE_PROGRESS) progress_end ();
		}
	}

	caetla_command (0x02);			// back to main menu ??
}

static void caetla_send_wave (u_long address, u_char *buffer, u_long size)
{
}

static void caetla_recv_wave (u_long address, u_char *buffer, u_long size)
{
}

static void caetla_go (ps_env_t *e, ps_header_t *h, bool console)
{
	assertz (e,"env");
	assertz (h,"header");

	caetla_command (0x16);

	rom_swap_32 (e->task);
	rom_swap_32 (e->event);
	rom_swap_32 (e->stack);

	rom_swap_32 (h->pc0);
	rom_swap_32 (h->gp0);
	rom_swap_32 (h->t_addr);
	rom_swap_32 (h->t_size);
	rom_swap_32 (h->d_addr);
	rom_swap_32 (h->d_size);
	rom_swap_32 (h->b_addr);
	rom_swap_32 (h->b_size);
	rom_swap_32 (h->s_addr);
	rom_swap_32 (h->s_size);
	rom_swap_32 (h->sp);
	rom_swap_32 (h->fp);
	rom_swap_32 (h->gp);
	rom_swap_32 (h->ret);
	rom_swap_32 (h->base);

	rom_swap_8 (0);
}

static void caetla_call(unsigned long address)
{
	caetla_command(0x17);
	rom_swap_32(address);
}

static void caetla_server (void)
{
	u_char x = 0;
	void *old_sig;

	old_sig = signal(SIGINT, control_c);
	printf("--- Console mode ---\n");
	caetla_command(4);
	rom_swap_8(1);
	while (caetla_server_quit == 0)
	{
		while (caetla_server_quit == 0)
		{
			x = rom_swap_8 ('P');
			if (x == 'd')
			{
				if (rom_swap_8('R') == 'o')
				{
					x = rom_swap_8 (0);
	/* Patch reflecting caetla 0.31 */
					if (x > 17) errorf ("caetla_server: %d\n",x);
					if (x == 17) {
						rom_swap_8(0);
						caetla_server_quit = 1;
						break;
					}
	/* ---------------------- */
					(*caetla_con[x])();
				}
			}
			else
			{
				if (x==10) usleep (20000);
			}
		}
	}
	signal(SIGINT, old_sig);
	caetla_command(4);
	rom_swap_8(0);
	printf("--- Console mode ended ---\n");
}

static void caetla_flash (u_char *buffer, u_long length)
{
	//="Licensed by Sony Computer Entertainment Inc.";
	u_short chk;
	u_char x;
	u_char *p;
	int i;

	caetla_listen();

	verbosef (VERBOSE_ALL,
	"FIRMWARE FLASHROM UPDATE. Switch PlayStation OFF and CTRL-C to abort!\n"
	"Press RESET on PlayStation if you want to update your flash ROM!\n"
	);

	while (rom_swap_8('W') != 'R');
	verbosef (VERBOSE_ALL,"Sending update program\n");
	while (rom_swap_8('B') != 'W');
	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8('X') != 'X');
	verbosef (VERBOSE_PROGRESS,"#");
	rom_swap_32 (0x80020000);
	rom_swap_32 (4100);

	p = (u_char *) flasher_rom;
	chk = 0;
	for (i=0;i<4100;i++)
	{
		if ((i % 256)==0) verbosef (VERBOSE_PROGRESS,"#");
		x = *p++;
		if (rom_swap_8 (x) != x) errorf("Transfer error!\n");
		chk += x;
	}
	rom_swap_16 ( chk & 0xfff );

	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8(0) != 'O');
	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8('W') != 'K');
	verbosef (VERBOSE_PROGRESS,"#\n");

	verbosef (VERBOSE_ALL,"Sending ROM image\n");

	while (rom_swap_8('W') != 'R');
	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8('B') != 'W');
	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8('U') != 'U');
	verbosef (VERBOSE_PROGRESS,"#");
	rom_swap_32 (0x80040000);
	rom_swap_32 (length);

	p = (u_char *) buffer;
	chk = 0;
	for (i=0;i<length;i++)
	{
		if ((i % 256)==0) verbosef (VERBOSE_PROGRESS,"#");
		x = *p++;
		if (rom_swap_8 (x) != x) errorf("Transfer error!\n");
		chk += x;
	}
	rom_swap_16 ( chk & 0xfff );

	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8(0) != 'O');
	verbosef (VERBOSE_PROGRESS,"#");
	while (rom_swap_8('W') != 'K');
	verbosef (VERBOSE_PROGRESS,"#\n");

	verbosef (VERBOSE_ALL,"Update complete.\n");
}

/* the memory card interface is not yet completed */

static void caetla_card_on (void)
{
	caetla_command (0x01);
	rom_swap_8 (1);
	rom_swap_8 (0);
}

static void caetla_card_off (void)
{
	caetla_command (0x02);
}

static void caetla_card_list (u_short slot,card_list_t *list)
{
	int i,j;

	caetla_command (0x17);
	rom_swap_8 (slot-1);
	list->count = rom_swap_16 (0);
	
	if (list->count>0)
	{
		for (i=0;i<list->count;i++)
		{
			u_char *p = (u_char *) &(list->entry[i]);
			for (j=0;j<128;j++) p[j] = rom_swap_8 (0);
		}
		rom_swap_32 (0);
	}
	else
	{
		rom_swap_16 (0);
	}
}

static void caetla_card_backup (u_short slot,u_short file, u_char *content)
{
	int j,k;

	caetla_command (0x18);
	rom_swap_8 (slot-1);
	rom_swap_8 (file);
	k = rom_swap_8 (0);

	verbose_if (VERBOSE_PROGRESS) progress_init (k*8192);
	for (j=0;j<(k*8192);j++)
	{
		content[j] = rom_swap_8(0);
		verbose_if (VERBOSE_PROGRESS) if ((j % 512)==0) progress_bar (j);
	}
	verbose_if (VERBOSE_PROGRESS) progress_end ();

	rom_swap_32 (0);
}

static void caetla_card_restore (u_short slot,u_short file, u_char *content, u_long size)
{
	int i;

//	caetla_command (0x1a);		// mc delete !!
//	rom_swap_8 (slot-1);
//	rom_swap_8 (file);
//	rom_swap_8 (0);

	caetla_command (0x19);
	rom_swap_8 (slot-1);

	// send filename, then size.

	i=0;
	while (content[i]) rom_swap_8 (content[i++]);
	rom_swap_8 (0);
	rom_swap_32 (size-54);
	rom_swap_8 (0);
	content += 54;

	verbose_if (VERBOSE_PROGRESS) progress_init (size-54);
	for (i=0;i<(size-54);i++)
	{
		rom_swap_8 (content[i]);
		verbose_if (VERBOSE_PROGRESS) if ((i % 512)==0) progress_bar (i);
	}
	verbose_if (VERBOSE_PROGRESS) progress_end ();

	rom_swap_32 (0);
}

static void caetla_card_delete (u_short slot, u_short file)
{
	caetla_command (0x1a);
	rom_swap_8 (slot-1);
	rom_swap_8 (file);
	rom_swap_16 (0);
}

static void caetla_card_format (u_short slot)
{
	caetla_command (0x1b);
	rom_swap_8 (slot-1);
	rom_swap_16 (0);
}

rom_module_t caetla_rom = {
	NULL,
	
	// module informations	
	"CAETLA",
	"0.9",
	"caetla rom driver",
	"The PIOCONS Team",
	#include "AUTHORS"

	,0x0,

	caetla_detect,
	caetla_reset,
	caetla_boot,
	caetla_send_memory,
	caetla_recv_memory,
	caetla_send_vram,
	caetla_recv_vram,
	NULL, //caetla_send_wave,
	NULL, //caetla_recv_wave,
	caetla_go,
	caetla_call,
	caetla_server,
	caetla_flash,

	caetla_card_on,
	caetla_card_off,
	caetla_card_list,
	caetla_card_backup,
	caetla_card_restore,
	caetla_card_delete,
	caetla_card_format,
};
