/* $Id: card.c,v 1.2 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************

	not yet finished. suggestions ?


 */

#include "card.h"
#include "error.h"
#include "rom.h"

static card_list_t card_list;

static char kanji_2_ascii [256] = {
//0  1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0x00
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0x10
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0x20
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0x30
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ','0',	// 0x40
'1','2','3','4','5','6','7','8','9',' ',' ',' ',' ',' ',' ',' ',	// 0x50
'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',	// 0x60
'Q','R','S','T','U','V','W','X','Y','Z',' ',' ',' ',' ',' ',' ',	// 0x70
'd','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',	// 0x80
'p','q','r','s','t','u','v','w','x','y','z',' ',' ',' ',' ',' ',	// 0x90
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0xA0
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0xB0
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0xC0
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0xD0
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0xE0
' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',	// 0xF0
};

static char *card_op_names[] =
{
	"BACKUP",
	"RESTORE",
	"LIST",
	"DELETE",
	"FORMAT",
};

char *card_name_of_op (u_short opcode)
{
	return card_op_names[opcode];
}

void card_operation (u_short slot, u_short opcode, char *filename)
{
	int i,len;
	pio_file_t *file;
	bool ok;

	rom_call (card_on);

	switch (opcode)
	{
		case MCOP_BACKUP:
			verbosef (VERBOSE_ALL,"Backup of memory card in slot %d:\n",slot);
			rom_call (card_list,slot,&card_list);

			for (i=0;i<card_list.count;i++)
			{
				if (filename==NULL)
				{
					ok = TRUE;
				}
				else
				{
					ok = (bool) 0==strcasecmp (filename,card_list.entry[i].name);
				}
			
				if (ok)
				{
					len = card_list.entry[i].size + 54;
					file = pio_file_openw (card_list.entry[i].name,len);
					if (file)
					{
						verbosef (VERBOSE_ALL,"Downloading %d bytes from \"%s\"\n",
							(int)card_list.entry[i].size,card_list.entry[i].name);
						strncpy (file->buffer,card_list.entry[i].name,54);
						rom_call (card_backup,slot,i,file->buffer+54);
						verbosef (VERBOSE_ALL,"\n");
						pio_file_close (file);
					}
					else errorf ("backup aborted\n");
				}
			}
			break;

		case MCOP_RESTORE:
			verbosef (VERBOSE_ALL,"Restore of memory card in slot %d:\n",slot);

			file = pio_file_openr (filename);

			verbosef (VERBOSE_ALL,"Uploading %d bytes to \"%s\"\n"
                               ,(int)file->length,filename);
			rom_call (card_restore,slot,0,file->buffer,file->length);
			verbosef (VERBOSE_ALL,"\n");
			pio_file_close (file);
			break;

		case MCOP_LIST:
			verbosef (VERBOSE_ALL,"List of memory card in slot %d:\n",slot);
			rom_call (card_list,slot,&card_list);

			if (card_list.count) verbosef (VERBOSE_ALL,
			"F#|B#|Filename            |Comment\n"
			"--+--+--------------------+-------------------------------\n");

			for (i=0;i<card_list.count;i++)
			{
				if (!(flags & FLAG_KANJI))
				{
					u_char *s = card_list.entry[i].comment;
					int i,j,l=strlen(s);
				
					for (i=0,j=0;i<l;i+=2,j+=1)
					{
						switch (s[i])
						{
							case 0x82:
							case 0x81:
								s[j] = kanji_2_ascii [s[i+1]];
								break;
							default:
								s[j] = '#';
								break;
						}
					}
					s[j]=0;
				
				}
			
				verbosef (VERBOSE_ALL,
					"%02d|%02d|%-20s|%-47s\n",i,
					(int)(card_list.entry[i].size/8192),
					card_list.entry[i].name,
					card_list.entry[i].comment);
			}
			break;

		case MCOP_DELETE:
			verbosef (VERBOSE_ALL,"Delete of memory card entry ..\n");
			rom_call (card_list,slot,&card_list);

			for (i=0;i<card_list.count;i++)
			{
				if (filename==NULL)
				{
					ok = TRUE;
				}
				else
				{
					ok = (bool) 0==strcasecmp (filename,card_list.entry[i].name);
				}
			
				if (ok)
				{
					rom_call (delete_card,slot,i);
					verbosef (VERBOSE_ALL,"Deleted \"%s\"\n",
							card_list.entry[i].name);
				}
			}
			break;

		case MCOP_FORMAT:
			verbosef (VERBOSE_ALL,"Format of memory card in slot %d:\n",slot);
			rom_call (format_card, slot);
			break;

		default:
			break;
	}

	rom_call (card_off);
}
