/* $Id: cpe.c,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************

	CPE file format loader.

 */

#include "pio.h"
#include "rom.h"
#include "error.h"

#define CPE_VERSION(x)	MAKE_ID('C','P','E',(x))
#define CPE_END			0x0

#define CPE_CHUNK		0x1
typedef struct {
	u_long	address;
	u_long	length;
	u_char	bytes[0];		// length bytes..
} cpe_chunk_t;

#define CPE_REGISTER	0x3
typedef struct {
	u_short	reg;
	u_long	val;
} cpe_register_t;

#define CPE_TARGET		0x8
typedef struct {
	u_char	target;
} cpe_target_t;

static bool cpe_detect (pio_file_t *file)
{
	u_char *p = file->buffer;

	if (*p++=='C') if (*p++=='P') if (*p++=='E') if (*p++==1) return TRUE;

	return FALSE;
}

static bool cpe_dump (pio_file_t *file)
{
	u_long addr,base,len;
	u_short reg;
	u_char *p = file->buffer + 4;
	bool cont = TRUE;

	verbosef (VERBOSE_ALL,"cpe_begin()\n");

	while (cont) switch (*p)
	{
		case CPE_REGISTER: {
			p++;
			reg = (((p)[1]<<8)|((p)[0])); p += 2;
			base = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			verbosef (VERBOSE_ALL,"cpe_register(%04x,%08lx)\n",reg,base);
		} break;

		case CPE_CHUNK: {
			p++;
			addr = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			len = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			verbosef (VERBOSE_ALL,"cpe_chunk(%08lx,%d)\n",addr,(int)len);
			p += len;
		} break;

		case CPE_TARGET: {
			p++;
			verbosef (VERBOSE_ALL,"cpe_target(%d)\n",*p);
			p++;
		} break;

		default:
			errorf ("unknown chunk %d\n",*p);
			break;

		case CPE_END:
			verbosef (VERBOSE_ALL,"cpe_end()\n");
			cont = FALSE;
			break;
	}
	
	return TRUE;
}

static bool cpe_send (pio_file_t *file)
{
	u_long addr,len;
	int snr=0;
	u_char *p = file->buffer + 4;
	bool cont = TRUE;

	rom_call (reset);

	verbosef (VERBOSE_ALL,"Object file format is \"CPE V1\"\n");

	while (cont) switch (*p)
	{
		case CPE_CHUNK: {
			p++;
			addr = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			len = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;

			verbosef (VERBOSE_PROGRESS,"Section [%8d] Address [%08lx] Length [%06lx] *\n",snr++,addr,len);
			rom_call (send_memory,addr,p,len);

			p += len;
		} break;

		case CPE_REGISTER:
			p+=3;
			addr = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			verbosef (VERBOSE_ALL,"PC=0x%08lx GP=0x%08lx\n",addr,0L);
			break;

		case CPE_TARGET:
			p += 2;
			break;

		default:
			errorf ("unknown chunk %d\n",*p);
			break;

		case CPE_END:
			cont = FALSE;
			break;
	}

	return TRUE;
}

static bool cpe_get_header (pio_file_t *file, ps_header_t *h)
{
	u_long least = 0xffffffff;
	u_long total = 0;
	u_long addr,base,len;
	u_short reg;
	u_char *p = file->buffer + 4;
	bool cont = TRUE;

	assertz (h,"header");

	h->pc0		= 0;
	h->gp0		= 0;
	h->t_addr	= 0; // least
	h->t_size	= 0; // total
	h->d_addr	= 0x801fff00;
	h->d_size	= 0;
	h->b_addr	= 0;
	h->b_size	= 0;
	h->s_addr	= 0;
	h->s_size	= 0;
	h->sp		= 0;
	h->fp		= 0;
	h->gp		= 0;
	h->ret		= 0;
	h->base		= 0;

	while (cont) switch (*p)
	{
		case CPE_REGISTER: {
			p++;
			reg = (((p)[1]<<8)|((p)[0])); p += 2;
			base = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			h->pc0 = base;
		} break;

		case CPE_CHUNK: {
			p++;
			addr = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			len = (((p)[3]<<24)|((p)[2]<<16)|(p[1]<<8)|(p[0])); p += 4;
			if (least>addr) least = addr;
			total += len;
			p += len;
		} break;

		case CPE_TARGET: {
			p += 2;
		} break;

		default:
			errorf ("unknown chunk %d\n",*p);
			break;

		case CPE_END:
			cont = FALSE;
			break;
	}

	h->t_addr = least;
	h->t_size = total;

	return TRUE;
}

pio_format_t cpe_pio = {
	NULL,
	
	// module informations	
	"CPE",
	"0.1",
	"CPE V1 format driver",
	"Daniel Balster <dbalster@uni-paderborn.de>",
	"Daniel Balster <dbalster@uni-paderborn.de>",

	NULL,
	cpe_detect,
	cpe_send,
	cpe_get_header,
	cpe_dump,
};
