/* $Id: defs.h,v 1.2 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#ifndef __DEFS_H__
#define __DEFS_H__

#define _GNU_SOURCE
#define _USE_GNU

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdarg.h>

#if defined(linux)
#include <getopt.h>
#include <sched.h>
#include <asm/io.h>
#endif

#if defined(__NetBSD__)
#include <machine/pio.h>
#include "glibc/getopt.h"
#endif

#if defined(__FreeBSD__)
#include <machine/cpufunc.h>
#include "glibc/getopt.h"
#endif

/* boolean type */
typedef enum { FALSE=0,TRUE=~(FALSE) } bool;

/* lo <= x <= hi */
#define rangeCheck(lo,x,hi) (((lo)<=(x))?(((x)<=(hi))?TRUE:FALSE):FALSE)

#define SWAP32(a) ( ((a >> 24) & 0xff) | ((a >> 8) & 0xff00) | ((a << 8) & 0xff0000) | ((a << 24) & 0xff000000) ) 
#define SWAP16(a) ( ((a >> 8) & 0xff) | ((a << 8) & 0xff00) )

/*
**	Debugger Macros. Use dprintf() for all kind of printf()'s that should only
**	be printed in debug mode. Use assertf() on ALL expressions which shouldn't
**	be checked each time or where continuous checks destroy the code flow.
**
**	note: use only during development.
*/

#if defined DEBUG
	#define debugf(fmt,args...)		fprintf (stderr,fmt, ## args)
	#define assertf(X,fmt,args...)	({ if(!(X)) fprintf (stderr,"\n*** ASSERTION FAILED: in %s(), file %s, line %d\n" ## fmt ## "\n\n",__FUNCTION__,__FILE__,__LINE__, ## args);})
	#define assert(X,msg)			({ if(!(X)) fprintf (stderr,"\n*** ASSERTION FAILED: in %s(), file %s, line %d\n" ## msg ## "\n\n",__FUNCTION__,__FILE__,__LINE__);})
	#define assertz(X,msg)			({ if((X)==0) fprintf (stderr,"\n*** ASSERTION FAILED: in %s(), file %s, line %d\n" ## msg ## "is a null pointer\n\n",__FUNCTION__,__FILE__,__LINE__);})
#else
	#define debugf(fmt,args...)
	#define assertf(X,args...)
	#define assert(X,msg)
	#define assertz(X,msg)
#endif
extern char *program_name;

/*
**	various flags. this keeps the settings more clear
*/

#define FLAG_REALTIME		(1L << 0)	// use more CPU time
#define FLAG_CONSOLE		(1L << 1)	// go into console mode
#define FLAG_DOWNLOAD		(1L << 2)	// download
#define FLAG_GO				(1L << 3)	// go
#define FLAG_INFO			(1L << 4)	// print informations
#define FLAG_KANJI			(1L << 5)	// enable kanji
#define FLAG_NICE			(1L << 6)	// be more nice
#define FLAG_RESET			(1L << 7)	// reset the ps
#define FLAG_CALL			(1L << 8)	// call an address
#define	FLAG_LIBPS			(1L << 9)	// patch to libps.exe
#define	FLAG_YAROZE			(1L << 10)	// patch to NetYaroze
extern u_long flags;

/*
**	various flags. this keeps the settings more clear
*/

enum {
	MODE_NORMAL,
	MODE_MEMORYCARD,
	MODE_IMAGE,
	MODE_FLASH,
	MODE_CODE,
	MODE_BOOT,
	MODE_CALL,
};
extern int mode;

/*
**	instead of using zillons of uncontrolled printf()'s, use verbosef() with
**	level controlled by the user. the higher the level the more printf()'s
**	should occur. Use this definitions:
**	maybe this is a little overkill..
*/

enum {
	VERBOSE_ALL=1,			// normal, not classified
	VERBOSE_PROGRESS,		// progress
	VERBOSE_RSRC,			// allocating/releasing resources
	VERBOSE_CHECK,			// check and detect phases
	VERBOSE_FIO,			// file I/O
	VERBOSE_PIO,			// port I/O
};

#define PORT_RW			(port+0)
#define PORT_STATUS		(port+2)
extern u_short port;

#define verbosef(lvl,fmt,args...)	if (lvl<=verbose_level) { printf(fmt, ## args);fflush(stdout); }
#define verbose_if(lvl)	if (lvl<=verbose_level) 
extern u_short verbose_level;

#define USER_WITH_IO_ACCESS	user_1
#define USER_WITH_NO_ACCESS	user_2
extern uid_t user_1;
extern uid_t user_2;

typedef struct {
	u_long pc0;
	u_long gp0;
	u_long t_addr;
	u_long t_size;
	u_long d_addr;
	u_long d_size;
	u_long b_addr;
	u_long b_size;
	u_long s_addr;
	u_long s_size;
	u_long sp,fp,gp,ret,base;
} ps_header_t;

typedef struct {
	char key[8];
	u_long text;
	u_long data;
	ps_header_t exec;
	char title[60];
} psx_header_t;

#define default_env(e) (e)->task=4,(e)->event=16,(e)->stack=0x801fff00
typedef struct {
	u_long task;
	u_long event;
	u_long stack;
} ps_env_t;

typedef void (*call_t)();

void progress_init (int maxlen);
void progress_bar (int k);
void progress_end (void);

#endif  __DEFS_H__
