/* $Id: pio.c,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************

	PIO is a wrapper for executable file formats and has utility functions.
	
	for read-only files, use pio_file_openr() (r = read)
	for write-only files, use pio_file_openw() (w = write)
	
	the returned struct contains a buffer, a length and a name. don't change!

 */

#include "pio.h"
#include "error.h"

static pio_format_t *pio_root;

void pio_add_format (pio_format_t *m)
{
	if (m->init) m->init();

	m->next = pio_root;
	pio_root = m;
}

pio_format_t *pio_detect (pio_file_t *pf)
{
	pio_format_t *pio = pio_root;
	
	verbosef (VERBOSE_CHECK,"autodetecting file format:\n");
	
	do {
		verbosef (VERBOSE_CHECK,"%s: ",pio->name);
		if (pio->detect(pf)) break;
		verbosef (VERBOSE_CHECK,"NO\n");
		pio = pio->next;
	} while (pio != NULL);

	if (pio) verbosef (VERBOSE_CHECK,"YES\n");

	return pio;
}

void pio_list (void)
{
	pio_format_t *pio = pio_root;
		
	do {
		verbosef (VERBOSE_ALL,"\"%s\" ",pio->name);
		pio = pio->next;
	} while (pio != NULL);

	verbosef (VERBOSE_ALL,"\n");
}


pio_file_t *pio_file_openr (char *filename)
{
	int fd;
	pio_file_t *pio = NULL;

	verbosef (VERBOSE_FIO,"Reading \"%s\"\n",filename);

	errno = 0;
	fd = open (filename,O_RDONLY);
	if (fd != -1)
	{
		int status;
		struct stat st;
	
		errno = 0;
		status = fstat (fd,&st);
		if (status != -1)
		{
			int bytes;
			
			errno = 0;
			bytes = (int) mmap (NULL,st.st_size,PROT_READ,MAP_PRIVATE,fd,0);
			if (bytes != -1)
			{
				pio = malloc (sizeof(pio_file_t));
				
				if (pio != NULL)
				{
					pio->type = PIO_READ;
					pio->name = strdup(filename);
					pio->buffer = (u_char *) bytes;
					pio->length = st.st_size;
				}
				else
				{
					munmap ((void*)bytes,st.st_size);
					warnf ("malloc(%d) failed\n", sizeof(pio_file_t));
				}
			}
			else warnf ("mmap() failed; %s\n", strerror(errno));
		}
		else warnf ("fstat() failed; %s\n", strerror(errno));
		close (fd);
	}
	else warnf ("open(%s) failed; %s\n",filename, strerror(errno));

	return pio;
}

pio_file_t *pio_file_openw (char *filename, u_long length)
{
	pio_file_t *pio = NULL;

	pio = malloc (sizeof(pio_file_t));
				
	if (pio != NULL)
	{
		pio->type = PIO_WRITE;
		pio->name = strdup(filename);
		pio->buffer = malloc (length);
		if (pio->buffer==NULL)
		{
			free (pio);
			pio = NULL;
			warnf ("malloc(%d) failed\n", length);
		}
		pio->length =length;
	}
	else warnf ("malloc(%d) failed\n", sizeof(pio_file_t));
		
	return pio;
}

void pio_file_close (pio_file_t *pf)
{
	if (pf)
	{
		switch (pf->type)
		{
			case PIO_READ:
				if (pf->buffer) munmap ((void*)pf->buffer,pf->length);
				break;
			case PIO_WRITE:
				if (pf->buffer)
				{
					FILE *file;
					
			//		verbosef (VERBOSE_FIO,"Writing \"%s\"\n",pf->name);

					file = fopen (pf->name,"wb");
					if (file)
					{
						if (1 != fwrite (pf->buffer,pf->length,1,file))
							warnf ("fwrite() failed; %s\n", strerror(errno));
						fclose (file);
					}
					else warnf ("open(%s) failed; %s\n",pf->name, strerror(errno));
					free (pf->buffer);
				}
				break;
		}
		if (pf->name) free (pf->name);
		free (pf);
	}
}
