/* $Id: pio.h,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#ifndef __PIO_H__
#define __PIO_H__

#include "defs.h"

enum {
	PIO_READ,
	PIO_WRITE
};

typedef struct {
	int type;				// PIO_READ or PIO_WRITE
	char *name;				// filename. readonly!
	u_char *buffer;			// pointer to length bytes
	u_long length;
} pio_file_t;

typedef struct _pio_format_t {
	struct _pio_format_t *next;

	char *name;				// "CPE"
	char *version;			// "1.0"
	char *description;		// "Psy-Q binary format"
	char *copyright;		// "Daniel Balster <dbalster@uni-paderborn.de>"
	char *author;			// "Daniel Balster <dbalster@uni-paderborn.de>"

	void (*init)(void);
	bool (*detect)(pio_file_t *file);
	bool (*send)(pio_file_t *file);
	bool (*get_header)(pio_file_t *file, ps_header_t *h);
	bool (*dump)(pio_file_t *file);

} pio_format_t;

/** prototypes **/

void pio_add_format (pio_format_t *m);
pio_format_t *pio_detect (pio_file_t *pf);
pio_file_t *pio_file_openr (char *filename);
pio_file_t *pio_file_openw (char *filename, u_long length);
void pio_file_close (pio_file_t *pf);
void pio_list (void);

#endif  __PIO_H__
