/* $Id: tim.c,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#include "vram.h"
#include "error.h"
#include "pio.h"

typedef struct {
	u_long id;
	u_long flag;
	u_long count;
	u_short x;
	u_short y;
	u_short w;
	u_short h;
} tim_header_t;

static bool tim_detect (char *filename)
{
	char *s = rindex(filename,'.');

	if (s==0) return FALSE;

	if (!strcasecmp(s,".tim")) return TRUE;

	return FALSE;
}

static void tim_write (char *filename, vram_image_t *image)
{
	pio_file_t *pio;
	u_long length;
	int i;
	
	length = sizeof(tim_header_t) + (image->w*image->h*(image->f + 2));

	pio = pio_file_openw (filename,length);
	if (pio)
	{
		tim_header_t *tim = (tim_header_t *) pio->buffer;
		u_short *pixel = (u_short *) (tim+1);
	
		tim->id		= 0x10;
		tim->flag	= image->f + 2;
		tim->count	= length - 8;
		tim->x		= image->x;
		tim->y		= image->y;
		tim->w		= image->f ? ((image->w/2)*3) : image->w;
		tim->h		= image->h;
		
		if (image->f==1)
		{
			u_char *p,*q;
			u_char r,g,b;
			
			p = (u_char *) pixel;
			q = (u_char *) image->pixel;
		
			for (i=0;i<(image->w*image->h);i++)
			{
				b = *q++; r = *q++; g = *q++;
				*p = r; p++; *p = g; p++; *p = b; p++;
			}
		}
		else memcpy (pixel,image->pixel,(image->w*image->h*2));

		pio_file_close (pio);
	}
	else errorf("vram download aborted\n");
}

static void tim_read (char *filename, vram_image_t *image)
{
	pio_file_t *pio;

	pio = pio_file_openr (filename);
	if (pio)
	{
		tim_header_t *tim = (tim_header_t *) pio->buffer;
		u_short *pixel = (u_short *) (tim+1);

		image->x = tim->x;
		image->y = tim->y;
		image->w = tim->w;
		image->h = tim->h;
		image->f = tim->flag-2;
		image->pixel = malloc (image->w*image->h*2);

		memcpy (image->pixel,pixel,(image->w*image->h*2));

		pio_file_close (pio);
	}
	else errorf("vram upload aborted\n");
}

vram_format_t tim_vram = {
	NULL,
	
	// module informations	
	"TIM",
	"0.2",
	"tim image driver",
	"Daniel Balster <dbalster@uni-paderborn.de>",
	"Daniel Balster <dbalster@uni-paderborn.de>",

	tim_detect,
	tim_write,
	tim_read,
};
